#include "Directory.h"
#include <dirent.h>
#include <stdio.h>
#include <sys/stat.h>

Vector<FileSystemPath> Directory::getSubdirectories(const FileSystemPath& path)
{
    Vector<FileSystemPath> ret;
    auto dirp = ::opendir(path.as_string().raw());
    while(auto ep = ::readdir(dirp))
    {
        auto rel_path = String(ep->d_name);
        if (rel_path == "." || rel_path == "..")
        {
            continue;
        }

        auto full_path = path.join(rel_path);
        struct stat sb;
        auto rc = lstat(full_path.as_string().raw(), &sb);
        if (rc == -1)
        {
            printf("Got error");
        }
        if (S_ISDIR(sb.st_mode))
        {
            printf("Adding full path: %s\n", full_path.as_string().raw());
            ret.push_back(full_path);
        }
    }
    ::closedir(dirp);
    return ret;
}

Vector<FileSystemPath> Directory::getFiles(const FileSystemPath& path, bool recursive)
{
    Vector<FileSystemPath> paths;
    if (path.is_directory())
    {
        for (const auto& entry : getSubdirectories(path))
        {
            if (entry.is_regular_file())
            {
                paths.push_back(entry);
            }
            else if(recursive && entry.is_directory())
            {
                const auto child_paths = getFiles(entry, recursive);
                paths.extend(child_paths);
            }
        }
    }
    return paths;
}

Vector<FileSystemPath> Directory::getFilesWithExtension(const FileSystemPath& path, const String& extension, bool recursive)
{
    Vector<FileSystemPath> paths;
    if (path.is_directory())
    {
        for (const auto& entry : getSubdirectories(path))
        {
            if (entry.is_regular_file() && entry.extension() == extension)
            {
                paths.push_back(entry);
            }
            else if(recursive && entry.is_directory())
            {
                const auto child_paths = getFilesWithExtension(entry, extension, recursive);
                paths.extend(child_paths);
            }
        }
    }
    return paths;
}

void Directory::create(const FileSystemPath& path, bool existsOk)
{
    (void)existsOk;
    FileSystemPath working_path;
    if (path.is_directory())
    {
        working_path = path;
    }
    else
    {
        working_path = path.parent_path();
    }

    if (!working_path.exists())
    {
        //std::filesystem::create_directories(working_path);
    }
}
