#pragma once

#include "Allocator.h"
#include <utility>

template<typename T>
class Ptr
{
public:
    Ptr()
    {
    }

    static Ptr create()
    {
        Ptr p;
        p.allocate();
        return std::move(p);
    }

    void allocate()
    {
        m_raw = m_allocator.allocate();
    }

    ~Ptr()
    {
        if (m_raw != nullptr)
        {
            m_allocator.do_delete(&m_raw);
        }
    }

    Ptr(const Ptr& other) = delete;

    Ptr(Ptr&& other)
        : m_allocator(std::move(other.m_allocator)),
            m_raw(other.m_raw)
    {
        other.m_raw = nullptr;
    }

    T* get()
    {
        return m_raw;
    }

    const T* get() const 
    {
        return m_raw;
    }

    T* operator->()
    {
        return m_raw;
    }

private:
    Allocator<T> m_allocator;
    T* m_raw{nullptr};
};