#pragma once

#include "PlatformImage.h"
#include "Pixel.h"

#include <memory>
#include <vector>

class Color;

template<typename T>
class SerializeableGrid;

class Image
{
public:
    using Index = std::pair<std::size_t, std::size_t>;
    using Indices = std::vector<Index>;

    Image(unsigned width, unsigned height, Color::Format format = Color::Format::RGBA);
    ~Image();
    static std::unique_ptr<Image> Create(unsigned width, unsigned height, Color::Format format = Color::Format::RGBA);

    unsigned getBytesPerRow() const;
    unsigned getWidth() const;
    unsigned getHeight() const;
    unsigned getBitDepth() const;
    unsigned getNumChannels() const;

    uint8_t getByte(std::size_t index) const;
    SerializeableGrid<Pixel>* getGrid() const;

    Color::Format getFormat() const;
    PlatformImage* getPlatformImage();

    void setPixelValues(const Indices& indices, const std::vector<Color>& colors);
    void setWidth(unsigned width);
    void setHeight(unsigned height);
    void setBitDepth(unsigned bitDepth);
    void setFormat(Color::Format format);

    void toBuffer(uint8_t* buffer, std::size_t bufferMaxSize);

private:
    void initialize();
    unsigned mWidth{1};
    unsigned mHeight{1};
    unsigned mBitDepth{8};
    Color::Format mFormat{Color::Format::RGBA};

    std::unique_ptr<SerializeableGrid<Pixel> > mGrid;
    std::unique_ptr<PlatformImage> mPlatformImage;
};
