#pragma once

#include "ByteTypes.h"

#include <string>
#include <sstream>

class PngInfo
{
public:
    enum class ColorType : Byte
    {
        GREYSCALE = 0,
        RGB = 2,
        PALETTE = 3,
        GREYSCALE_ALPHA = 4,
        RGB_ALPHA = 6
    };

    enum class CompressionMethod : Byte
    {
        DEFLATE = 0
    };

    enum class FilterMethod : Byte
    {
        ADAPTIVE = 0
    };

    enum class InterlaceMethod : Byte
    {
        NONE = 0,
        ADAM7 = 1
    };

    ColorType getColorType() const;

    std::string toString(ColorType colorType) const;

    std::string toString(CompressionMethod method) const;

    std::string toString(FilterMethod method) const;

    std::string toString(InterlaceMethod method) const;

    bool bitDepthIsValid(ColorType colorType, Byte bitDepth) const;

    bool compressionMethodIsValid(Byte method);

    bool filterMethodIsValid(Byte method);

    std::string toString() const;

    ColorType mColorType{ColorType::RGB};
    CompressionMethod mCompressionMethod{CompressionMethod::DEFLATE};
    FilterMethod mFilterMethod{FilterMethod::ADAPTIVE};
    InterlaceMethod mInterlaceMethod{InterlaceMethod::NONE};
};




